"""
    Agilicus API

    Agilicus is API-first. Modern software is controlled by other software, is open, is available for you to use the way you want, securely, simply.  The OpenAPI Specification in YAML format is available on [www](https://www.agilicus.com/www/api/agilicus-openapi.yaml) for importing to other tools.  A rendered, online viewable and usable version of this specification is available at [api](https://www.agilicus.com/api). You may try the API inline directly in the web page. To do so, first obtain an Authentication Token (the simplest way is to install the Python SDK, and then run `agilicus-cli --issuer https://MYISSUER get-token`). You will need an org-id for most calls (and can obtain from `agilicus-cli --issuer https://MYISSUER list-orgs`). The `MYISSUER` will typically be `auth.MYDOMAIN`, and you will see it as you sign-in to the administrative UI.  This API releases on Bearer-Token authentication. To obtain a valid bearer token you will need to Authenticate to an Issuer with OpenID Connect (a superset of OAUTH2).  Your \"issuer\" will look like https://auth.MYDOMAIN. For example, when you signed-up, if you said \"use my own domain name\" and assigned a CNAME of cloud.example.com, then your issuer would be https://auth.cloud.example.com.  If you selected \"use an Agilicus supplied domain name\", your issuer would look like https://auth.myorg.agilicus.cloud.  For test purposes you can use our [Python SDK](https://pypi.org/project/agilicus/) and run `agilicus-cli --issuer https://auth.MYDOMAIN get-token`.  This API may be used in any language runtime that supports OpenAPI 3.0, or, you may use our [Python SDK](https://pypi.org/project/agilicus/), our [Typescript SDK](https://www.npmjs.com/package/@agilicus/angular), or our [Golang SDK](https://git.agilicus.com/pub/sdk-go).  100% of the activities in our system our API-driven, from our web-admin, through our progressive web applications, to all internals: there is nothing that is not accessible.  For more information, see [developer resources](https://www.agilicus.com/developer).   # noqa: E501

    The version of the OpenAPI document: 2024.12.18
    Contact: dev@agilicus.com
    Generated by: https://openapi-generator.tech
"""


import unittest

import agilicus_api
from agilicus_api.api.trusted_certs_api import TrustedCertsApi  # noqa: E501


class TestTrustedCertsApi(unittest.TestCase):
    """TrustedCertsApi unit test stubs"""

    def setUp(self):
        self.api = TrustedCertsApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_create_bundle(self):
        """Test case for create_bundle

        Creates a TrustedCertificateBundle  # noqa: E501
        """
        pass

    def test_create_label(self):
        """Test case for create_label

        Creates a TrustedCertificateLabel  # noqa: E501
        """
        pass

    def test_create_trusted_cert(self):
        """Test case for create_trusted_cert

        Creates a TrustedCertificate  # noqa: E501
        """
        pass

    def test_delete_bundle(self):
        """Test case for delete_bundle

        Delete a TrustedCertificateBundle  # noqa: E501
        """
        pass

    def test_delete_label(self):
        """Test case for delete_label

        Delete a TrustedCertificateLabel  # noqa: E501
        """
        pass

    def test_delete_trusted_cert(self):
        """Test case for delete_trusted_cert

        Delete a TrustedCertificate  # noqa: E501
        """
        pass

    def test_get_bundle(self):
        """Test case for get_bundle

        Get a TrustedCertificateBundle  # noqa: E501
        """
        pass

    def test_get_label(self):
        """Test case for get_label

        Get a TrustedCertificateLabel  # noqa: E501
        """
        pass

    def test_get_trusted_cert(self):
        """Test case for get_trusted_cert

        Get a TrustedCertificate  # noqa: E501
        """
        pass

    def test_list_bundles(self):
        """Test case for list_bundles

        list TrustedCertificateBundle  # noqa: E501
        """
        pass

    def test_list_cert_orgs(self):
        """Test case for list_cert_orgs

        list orgs that have available certificates  # noqa: E501
        """
        pass

    def test_list_labels(self):
        """Test case for list_labels

        list TrustedCertificateLabel  # noqa: E501
        """
        pass

    def test_list_trusted_certs(self):
        """Test case for list_trusted_certs

        list certificates  # noqa: E501
        """
        pass

    def test_replace_bundle(self):
        """Test case for replace_bundle

        Update a TrustedCertificateBundle  # noqa: E501
        """
        pass

    def test_replace_trusted_cert(self):
        """Test case for replace_trusted_cert

        Update a TrustedCertificate  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



def find_item(items, key, keyval):
    for item in items:
        if item[key] == keyval:
            return item
    return None


def find_object(items, key, keyval):
    for item in items:
        if getattr(item, key, None) == keyval:
            return item
    return None



from unittest.mock import MagicMock

class MessagesApiMock:

    def __init__(self):
        self.mock_bulk_delete_messages = MagicMock()
        self.mock_create_message = MagicMock()
        self.mock_create_routed_message = MagicMock()
        self.mock_create_user_message = MagicMock()
        self.mock_delete_inbox_item = MagicMock()
        self.mock_delete_message_endpoint = MagicMock()
        self.mock_get_inbox_item = MagicMock()
        self.mock_get_message_endpoint = MagicMock()
        self.mock_list_inbox_items = MagicMock()
        self.mock_list_message_endpoints = MagicMock()
        self.mock_list_messages_config = MagicMock()
        self.mock_replace_inbox_item = MagicMock()
        self.mock_replace_message_endpoint = MagicMock()
        self.mock_update_message_endpoint = MagicMock()

    def bulk_delete_messages(self, *args, **kwargs):
        """
        This method mocks the original api MessagesApi.bulk_delete_messages with MagicMock.
        """
        return self.mock_bulk_delete_messages(self, *args, **kwargs)

    def create_message(self, *args, **kwargs):
        """
        This method mocks the original api MessagesApi.create_message with MagicMock.
        """
        return self.mock_create_message(self, *args, **kwargs)

    def create_routed_message(self, *args, **kwargs):
        """
        This method mocks the original api MessagesApi.create_routed_message with MagicMock.
        """
        return self.mock_create_routed_message(self, *args, **kwargs)

    def create_user_message(self, *args, **kwargs):
        """
        This method mocks the original api MessagesApi.create_user_message with MagicMock.
        """
        return self.mock_create_user_message(self, *args, **kwargs)

    def delete_inbox_item(self, *args, **kwargs):
        """
        This method mocks the original api MessagesApi.delete_inbox_item with MagicMock.
        """
        return self.mock_delete_inbox_item(self, *args, **kwargs)

    def delete_message_endpoint(self, *args, **kwargs):
        """
        This method mocks the original api MessagesApi.delete_message_endpoint with MagicMock.
        """
        return self.mock_delete_message_endpoint(self, *args, **kwargs)

    def get_inbox_item(self, *args, **kwargs):
        """
        This method mocks the original api MessagesApi.get_inbox_item with MagicMock.
        """
        return self.mock_get_inbox_item(self, *args, **kwargs)

    def get_message_endpoint(self, *args, **kwargs):
        """
        This method mocks the original api MessagesApi.get_message_endpoint with MagicMock.
        """
        return self.mock_get_message_endpoint(self, *args, **kwargs)

    def list_inbox_items(self, *args, **kwargs):
        """
        This method mocks the original api MessagesApi.list_inbox_items with MagicMock.
        """
        return self.mock_list_inbox_items(self, *args, **kwargs)

    def list_message_endpoints(self, *args, **kwargs):
        """
        This method mocks the original api MessagesApi.list_message_endpoints with MagicMock.
        """
        return self.mock_list_message_endpoints(self, *args, **kwargs)

    def list_messages_config(self, *args, **kwargs):
        """
        This method mocks the original api MessagesApi.list_messages_config with MagicMock.
        """
        return self.mock_list_messages_config(self, *args, **kwargs)

    def replace_inbox_item(self, *args, **kwargs):
        """
        This method mocks the original api MessagesApi.replace_inbox_item with MagicMock.
        """
        return self.mock_replace_inbox_item(self, *args, **kwargs)

    def replace_message_endpoint(self, *args, **kwargs):
        """
        This method mocks the original api MessagesApi.replace_message_endpoint with MagicMock.
        """
        return self.mock_replace_message_endpoint(self, *args, **kwargs)

    def update_message_endpoint(self, *args, **kwargs):
        """
        This method mocks the original api MessagesApi.update_message_endpoint with MagicMock.
        """
        return self.mock_update_message_endpoint(self, *args, **kwargs)




import copy
import datetime
import json
import os
from ..input_helpers import (
    get_org_from_input_or_ctx,
    model_from_dict,
    strip_none,
)

from .. import context

from agilicus import (
    create_or_update,
    LabelName,
    PolicyTemplateInstance,
    PolicyTemplateInstanceSpec,
    MFAPolicyTemplate,
    SourceInfoPolicyTemplate,
    SimpleResourcePolicyTemplateStructureNode,
    SimpleResourcePolicyTemplateStructure,
    SimpleResourcePolicyTemplate,
    StandaloneRuleName,
    RuleAction,
    ResourceConfig,
    RulesConfig,
    RuleCondition,
    HttpRuleCondition,
    EmptiableObjectType,
)

from ..output.table import (
    format_table,
    spec_column,
    metadata_column,
    subtable,
    column,
)
from ..resources import query_resources
from ..resources import reconcile_default_policy


class InstanceAddInfo:
    def __init__(self, apiclient):
        super().__init__()


def set_multifactor_policy(ctx, name, duration, label=None, description=None, **kwargs):
    org_id = get_org_from_input_or_ctx(ctx, **kwargs)

    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    mfa = MFAPolicyTemplate(
        seconds_since_last_challenge=duration,
        labels=[LabelName(la) for la in (label or [])],
        template_type="mfa",
    )

    spec = PolicyTemplateInstanceSpec(
        org_id=org_id,
        name=name,
        template=mfa,
    )

    if description is not None:
        spec.description = description

    tmpl = PolicyTemplateInstance(spec=spec)
    templates_api = apiclient.policy_templates_api
    resp, _ = create_or_update(
        tmpl,
        lambda obj: templates_api.create_policy_template_instance(obj),
        lambda guid, obj: templates_api.replace_policy_template_instance(
            guid, policy_template_instance=obj
        ),
        to_dict=False,
    )
    return resp


def ruleset_labelled(ruleset, label):
    for ruleset_label in ruleset.spec.labels or []:
        if str(ruleset_label) == label:
            return True
    return False


def list_multifactor_policies(ctx, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    result = apiclient.policy_templates_api.list_policy_template_instances(
        org_id=org_id, template_type="mfa"
    )
    return result.policy_template_instances


def format_multifactor_policies(ctx, templates):
    mfa_columns = [
        column("seconds_since_last_challenge"),
        column("labels"),
    ]
    mfa_table = subtable(ctx, "spec.template", mfa_columns)
    columns = [
        spec_column("org_id"),
        spec_column("name"),
        spec_column("template.template_type", "type"),
        mfa_table,
    ]

    return format_table(ctx, templates, columns)


def list_policy_templates(ctx, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    kwargs["org_id"] = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs = strip_none(kwargs)
    result = apiclient.policy_templates_api.list_policy_template_instances(**kwargs)
    return result.policy_template_instances


def delete_policy_template(ctx, instance_id, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    kwargs["org_id"] = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs = strip_none(kwargs)
    apiclient.policy_templates_api.delete_policy_template_instance(instance_id, **kwargs)


def get_policy_template(ctx, instance_id, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    kwargs["org_id"] = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs = strip_none(kwargs)
    return apiclient.policy_templates_api.get_policy_template_instance(
        instance_id, **kwargs
    )


def format_policy_templates(ctx, templates):
    columns = [
        spec_column("org_id"),
        metadata_column("id"),
        spec_column("name"),
        spec_column("template.template_type", "type"),
        spec_column("description"),
        spec_column("template"),
    ]

    return format_table(ctx, templates, columns)


def set_source_info_policy(
    ctx,
    name,
    action,
    source_subnet,
    iso_country_code,
    invert,
    log_message=None,
    label=None,
    description=None,
    **kwargs,
):
    org_id = get_org_from_input_or_ctx(ctx, **kwargs)

    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    tmpl = SourceInfoPolicyTemplate(
        action=action,
        source_subnets=list(source_subnet or []),
        iso_country_codes=list(iso_country_code or []),
        invert=invert,
        labels=[LabelName(la) for la in (label or [])],
        template_type="source_info",
    )

    if log_message:
        tmpl.log_message = log_message

    spec = PolicyTemplateInstanceSpec(
        org_id=org_id,
        name=name,
        template=tmpl,
    )

    if description is not None:
        spec.description = description

    tmpl = PolicyTemplateInstance(spec=spec)
    templates_api = apiclient.policy_templates_api
    resp, _ = create_or_update(
        tmpl,
        lambda obj: templates_api.create_policy_template_instance(obj),
        lambda guid, obj: templates_api.replace_policy_template_instance(
            guid, policy_template_instance=obj
        ),
        to_dict=False,
    )
    return resp


def migrate_policy_rules(ctx, org_id=None, dump_dir=None, **kwargs):
    org_id = get_org_from_input_or_ctx(ctx, org_id=org_id)
    kwargs = strip_none(kwargs)
    if not kwargs.get("resource_id"):
        print("  migrating all applications")
    for res in query_resources(
        ctx, resource_type="application", org_id=org_id, **kwargs
    ):
        migrate_resource(ctx, res, dump_dir=dump_dir)


def fetch_resource_rules(ctx, org_id=None, dump_dir=None, **kwargs):
    org_id = get_org_from_input_or_ctx(ctx, org_id=org_id)
    kwargs = strip_none(kwargs)
    if not kwargs.get("resource_id"):
        print("  fetching all applications")
    for res in query_resources(
        ctx, resource_type="application", org_id=org_id, **kwargs
    ):
        fetch_resource(ctx, res, dump_dir=dump_dir)


def migrate_resource(ctx, resource, dump_dir=None):
    print(f"  migrating resource: {resource.spec.name}")
    resource.spec.config = resource.spec.config or ResourceConfig()
    resource.spec.config.rules_config = (
        resource.spec.config.rules_config or RulesConfig()
    )

    rules = resource.spec.config.rules_config.rules or []
    policy_structures = []
    for rule in rules:
        node = SimpleResourcePolicyTemplateStructureNode(
            priority=rule.priority or 0,
            rule_name=StandaloneRuleName(rule.name),
            children=[],
        )
        policy_structures.append(
            SimpleResourcePolicyTemplateStructure(
                name=StandaloneRuleName(rule.name),
                root_node=node,
            )
        )
    template = SimpleResourcePolicyTemplate(
        rules=[_migrate_http_rule(rule) for rule in rules],
        policy_structure=policy_structures,
        template_type="simple_resource",
    )
    instance_spec = PolicyTemplateInstanceSpec(
        org_id=resource.spec.org_id,
        template=template,
        name="resource-policy",
        object_id=resource.metadata.id,
        object_type=EmptiableObjectType(resource.spec.resource_type.value),
    )

    tmpl = PolicyTemplateInstance(
        spec=instance_spec,
    )

    # ensure the resource has been reconciled
    resource = reconcile_default_policy(
        ctx, resource.metadata.id, org_id=resource.spec.org_id
    )

    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    templates_api = apiclient.policy_templates_api

    def do_update(guid, obj):
        return templates_api.replace_policy_template_instance(
            guid, policy_template_instance=obj
        )

    updater = do_update
    # If there are no rules, don't blow away any previously create config. This allows us
    # to reapply the policy from scratch using a new config if needed, but at the same
    # time, ensures we don't blow away config if we run the same migration twice (since
    # the first migration will clear the rules, meaning the second will set an empty
    # policy. )
    if not rules:
        updater = None
    resp, _ = create_or_update(
        tmpl,
        lambda obj: templates_api.create_policy_template_instance(obj),
        updater,
        to_dict=False,
    )

    if rules:
        _clear_old_rules(apiclient, resource, dump_dir)
    return resp


def fetch_resource(ctx, resource, dump_dir=None):
    print(f"  fetching resource: {resource.spec.name}")
    resource.spec.config = resource.spec.config or ResourceConfig()
    resource.spec.config.rules_config = (
        resource.spec.config.rules_config or RulesConfig()
    )

    if dump_dir:
        _dump_resource_rules(resource, dump_dir)
    else:
        print(resource.spec.config.rules_config.to_dict())


def _dump_resource_rules(resource, dump_dir):
    now = datetime.datetime.now().strftime("%Y-%m-%dT%H-%M-%S")
    file_name = f"{resource.spec.org_id}-{resource.spec.name}-{now}.json"
    with open(os.path.join(dump_dir, file_name), "w") as f:
        json.dump(resource.spec.config.rules_config.to_dict(), f)


def _clear_old_rules(apiclient, resource, dump_dir):
    if dump_dir:
        _dump_resource_rules(resource, dump_dir)

    resource.spec.config.rules_config = RulesConfig(rules=[])

    apiclient.resources_api.replace_resource(resource.metadata.id, resource=resource)


def _migrate_http_rule(input_rule):
    if input_rule.extended_condition is not None:
        return input_rule

    new_rule = copy.deepcopy(input_rule)
    new_rule.actions = new_rule.actions or [RuleAction(action="allow")]
    input_cond = new_rule.condition.to_dict()
    input_cond["condition_type"] = "http_rule_condition"
    cond = model_from_dict(HttpRuleCondition, input_cond)
    new_rule.extended_condition = RuleCondition(
        condition=cond,
        negated=False,
    )
    del new_rule["condition"]
    return new_rule


def create_policy_template(ctx, template_dict, **kwargs):
    template_as_instance = model_from_dict(PolicyTemplateInstance, template_dict)
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    return apiclient.policy_templates_api.create_policy_template_instance(
        template_as_instance
    )


def replace_policy_template(ctx, instance_id, template_dict, **kwargs):
    template_as_instance = model_from_dict(PolicyTemplateInstance, template_dict)
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    return apiclient.policy_templates_api.replace_policy_template_instance(
        instance_id, policy_template_instance=template_as_instance
    )


def kick_policy_template(ctx, instance_id, org_id=None, **kwargs):
    org_id = get_org_from_input_or_ctx(ctx, org_id=org_id)
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    existing = apiclient.policy_templates_api.get_policy_template_instance(
        instance_id,
        org_id=org_id,
    )
    return apiclient.policy_templates_api.replace_policy_template_instance(
        instance_id,
        policy_template_instance=existing,
    )



from agilicus.agilicus_api import (
    ResourceConfig,
)


def map_resource_published(mapping, published):
    if published is not None:
        config = mapping.spec.resource_config
        if config is None:
            config = ResourceConfig()
            mapping.spec.resource_config = config
        mapping.spec.resource_config["published"] = published
    return mapping



import click

from . import public_file_org_links
from ..output.table import output_entry


@click.command(name="add-public-file-org-link")
@click.option("--link-org-id", required=True)
@click.option("--target-org-id", required=True)
@click.option("--file-tag", required=True)
@click.pass_context
def cli_command_add_public_file_org_link(ctx, **kwargs):
    output_entry(ctx, public_file_org_links.add_public_file_org_link(ctx, **kwargs))


@click.command(name="list-public-file-org-links")
@click.option("--link-org-id", default=None)
@click.option("--tag", default=None)
@click.option("--page-at-id", default=None)
@click.pass_context
def cli_command_list_public_file_org_links(ctx, **kwargs):
    result = public_file_org_links.list_public_file_org_links(ctx, **kwargs)
    print(public_file_org_links.format_public_file_org_links(ctx, result))


@click.command(name="get-public-file-org-link")
@click.option("--public-file-org-link-id", required=True)
@click.option("--link-org-id", required=True)
@click.pass_context
def cli_command_get_public_file_org_link(ctx, **kwargs):
    output_entry(
        ctx, public_file_org_links.get_public_file_org_link(ctx, **kwargs).to_dict()
    )


@click.command(name="delete-public-file-org-link")
@click.option("--public-file-org-link-id", required=True)
@click.option("--link-org-id", required=True)
@click.pass_context
def cli_command_delete_public_file_org_link(ctx, **kwargs):
    public_file_org_links.delete_public_file_org_link(ctx, **kwargs)


all_funcs = [func for func in dir() if "cli_command_" in func]


def add_commands(cli):
    glob = globals()
    for func in all_funcs:
        cli.add_command(glob[func])
